/* 
 *
 * Copyright (c) 2004-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 */



#define NONSPEC 1
#include <nonspec.h>
#include <dsu.h>
#include <caml/mlvalues.h>
#include <caml/memory.h>

static int * scratchpad = 0;
static int scratchpad_length = 0;
static region sregion = NULL;

/* #define BANSHEE_IF_C_DEBUG 1 */
/* #define BANSHEE_IF_C_DEBUG2 1 */

value banshee_make_const_node(value tag) {
  dsu_node d;

#ifdef BANSHEE_IF_C_DEBUG
  printf("banshee_make_const_node...\n");
#endif

  CAMLparam1(tag);
  d = make_const_dsu_node(String_val(tag));

#ifdef BANSHEE_IF_C_DEBUG
  printf("banshee_make_const_node done\n");
#endif

  CAMLreturn((value)d);
}

value banshee_make_var_node(value tag) {
  dsu_node d;

#ifdef BANSHEE_IF_C_DEBUG
  printf("banshee_make_var_node...\n");
#endif

  CAMLparam1(tag);
  d = make_var_dsu_node(String_val(tag));

#ifdef BANSHEE_IF_C_DEBUG
  printf("banshee_make_var_node done\n");
#endif

  CAMLreturn((value)d);
}

value banshee_subtype_edge(value t1, value t2) {
  dsu_node d1, d2;
  CAMLparam2(t1, t2);

#ifdef BANSHEE_IF_C_DEBUG
  printf("banshee_subtype_edge...\n");
#endif

  d1 = (dsu_node) t1;
  d2 = (dsu_node) t2;
  make_dsu_subtype_edge(d1,d2);

#ifdef BANSHEE_IF_C_DEBUG
  printf("banshee_subtype_edge done\n");
#endif

  CAMLreturn(Val_unit);
}

value banshee_initcfl(value nr_abs_types) {
  CAMLparam1(nr_abs_types);

#ifdef BANSHEE_IF_C_DEBUG2
  printf("banshee_initcfl...\n");
#endif

  nonspec_init();

  sregion = newregion();
  scratchpad_length = Int_val(nr_abs_types);
  scratchpad = ralloc(sregion, int[scratchpad_length]);

#ifdef BANSHEE_IF_C_DEBUG2
  printf("scratchpad ralloc'd, scratchpad_length = %d\n", scratchpad_length);
#endif

  dsu_init();
  flag_merge_projections = TRUE;
/*   flag_merge_projections = FALSE; */

#ifdef BANSHEE_IF_C_DEBUG
  printf("banshee_initcfl done\n");
#endif

  CAMLreturn(Val_unit);
}

value banshee_done_building(value unit) {
  CAMLparam1(unit);
  dsu_finished_adding();
  CAMLreturn(Val_unit);
}

value banshee_reaches(value n1, value n2) {
  dsu_node d1, d2;
  CAMLparam2(n1, n2);
  d1 = (dsu_node) n1;
  d2 = (dsu_node) n2;

  if(dsu_check_reaches(d1, d2)) CAMLreturn(Val_true);

  CAMLreturn(Val_false);
}

value banshee_sweep_reachable_types(value d, value v) {
  int i;
  dsu_node nd;

#ifdef BANSHEE_IF_C_DEBUG
  printf("banshee_sweep_reachable_types...\n");
#endif

  CAMLparam2(d,v);
  nd = (dsu_node) d;

  for(i = 0; i < scratchpad_length; i++)
    scratchpad[i] = 0;

  dsu_sweep_reachable_consts(nd,scratchpad);

  for(i = 0; i < scratchpad_length; i++)
    { 
      if (scratchpad[i] == 0)
        Store_field(v,i,Val_int(0));
      else
        Store_field(v,i,Val_int(1));
    }

#ifdef BANSHEE_IF_C_DEBUG2
  printf("Reachable types (%s) : ", (char *)((void*)nd+8));
    for(i = 0; i < scratchpad_length; i++) 
      printf(" %d",scratchpad[i]);

  printf("\nbanshee_sweep_reachable_types done\n");
#endif

  CAMLreturn(Val_unit);
}

extern void engine_stats(FILE*);

void dsu_banshee_stats()
{
  engine_stats(stderr);
}
void banshee_reset()
{
  deleteregion(sregion);
  dsu_reset();
  nonspec_reset();
}
