(* 
 *
 * Copyright (c) 2004-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)




(* 
   All named types that are used as bitfields in structures must be made non-updatable.
   This is because we can not pass bit fields by reference, as we do when concretising them.
   Also, using abstract types as bit fields exposes some of their structure and so they are 
   not abstract any more!
*)

open Cil

class bitfieldPinDownVisitor : cilVisitor = object(self)
  inherit nopCilVisitor

  method vtype t =
     match t with 
        TComp(ci,attr) ->
           let pindown_iter fi = 
              begin match fi.fbitfield with
                 None -> ()
               | Some _ -> 
                  if Namedtypes.isAbstractType fi.ftype 
                  then Namedtypes.setTypeNonUpdatable fi.ftype !(Cil.currentLoc) "used a bit field"
              end 
           in
           List.iter pindown_iter ci.cfields;
           DoChildren

      | _ -> DoChildren
              
end;;
