(* 
 *
 * Copyright (c) 2008-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)

open Cil

module H = Hashtbl

(* insert updates in a statement *)

class wrapBlockingCallsVisitor startBlockingFun endBlockingFun blockingFunNames : cilVisitor = object(self)
  inherit nopCilVisitor
    
  method vinst i = match i with
      Call(_, Lval(Var vi, NoOffset), _, _) when H.mem blockingFunNames vi.vname -> 
        let loc = !Cil.currentLoc in (* this is valid since we're in a visitor *)
        let startBCall = Call(None, Lval(var startBlockingFun), [], loc) in
        let endBCall = Call(None, Lval(var endBlockingFun), [], loc) in
            ChangeTo ([startBCall; i; endBCall])

        
    | _ -> SkipChildren
end


let wrapBlockingCalls startBlockingFun endBlockingFun (file : Cil.file) : unit = 
  if Updateargs.args.Updateargs.arg_wrapBlockingCalls
  then
    let blockingFunNames = H.create 13 in
    List.iter (fun name -> H.add blockingFunNames name ()) Updateconfig.predefinedBlockingCalls;  
    visitCilFile (new wrapBlockingCallsVisitor startBlockingFun endBlockingFun blockingFunNames) file;
;;
