(* 
 *
 * Copyright (c) 2004-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *  Gareth Stoyle       <gareth.stoyle@cl.cam.ac.uk> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



(* special directive to allow the user to specify malloc functions *)

open Cil
module H = Hashtbl

type mallocHash_t = (string, unit) H.t


let isMallocFunc (mallocHash : mallocHash_t) (fname:string) = 
   H.mem mallocHash fname



(* Collect all of the malloc pragmas in the file and note the names of the malloc functions *)
let collectMallocPragmas (file:Cil.file) : mallocHash_t =
   let mallocHash : mallocHash_t = H.create 7 in 
    
   let processPragma (g : Cil.global) : unit =
      match g with 
         GPragma(Attr (name,[AStr malloc_name]),loc) when name = Updateconfig.mallocPragmaName ->
            H.add mallocHash malloc_name ()

       | _ -> ()
   in
   (* first add the 'pre-defined' mallocs *)   
   List.iter (fun x -> H.add mallocHash x ()) Updateconfig.predefinedMallocs;
    
   (* then take care of the programmer-inserted ones *)   
   Cil.iterGlobals file processPragma;
   mallocHash
