(* 
 *
 * Copyright (c) 2004-2009, 
 * 
 *  Michael Hicks       <mwh@cs.umd.edu> 
 *  Gareth Stoyle       <gareth.stoyle@cl.cam.ac.uk> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



(* Functions in the C-library that we have to treat specially *)

open Cil

module H = Hashtbl


type sf_data_t = 
   {
     sf_num_args       : int;        (* number of arguments *)
     sf_args_to_elide  : int list;   (* the index of the arguments not to process - e.g. 1,2 for memcpy because
                                        they induce void* casts, but we know they are safe wrt update*)
     sf_elide_ret      : bool;       (* whether to process the return - e.g. false for memcpy *)
   }


(* The functions that we should treat differently *)
let special_functions = 
   [ 
     (* sf_name, {sf_num_args = 0; sf_args_to_elide = []; sf_elide_ret = true}*)
     "memcpy", {sf_num_args = 3; sf_args_to_elide = [0;1]; sf_elide_ret = true};
     "malloc", {sf_num_args = 1; sf_args_to_elide = [0];   sf_elide_ret = true};
   ]


(* hash from special function names to the information about how to process that function *)
let sf_hash : (string, sf_data_t) H.t = 
   let hash = H.create 4099 in 
   List.iter (fun (name,sf) -> H.add hash name sf) special_functions;
   hash



let isSpecialFun (fname:string) : bool = H.mem sf_hash fname


(* Should we elide processing of argument arg_index of function fname.
   Raise failure if fname not a special function *)
let shouldElide (fname:string) (arg_index : int) : bool = 
   List.mem arg_index (H.find sf_hash fname).sf_args_to_elide


(* Should we process the return of this special function  *)
let shouldElideReturn (fname:string) : bool = 
   (H.find sf_hash fname).sf_elide_ret
