(* 
 *
 * Copyright (c) 2004-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *  Michael Hicks       <mwh@cs.umd.edu> 
 *  Gareth Stoyle       <gareth.stoyle@cl.cam.ac.uk> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



type mode = TestMode | NormalMode | PatchMode | PrintVDMode
type updatePointsPolicy = UNone | UReturnonly | UFull | UEndOfTx
type updatabilityPolicy = UPNone | UPTx
type effectPrintingPolicy = EPNone | EPTypes | EPFunctions | EPSolveOnly | EPDifferential | EPFull
type unsoundnessAction = UnsoundWarn | UnsoundIgnore | UnsoundKillTypeShallow | UnsoundKillTypeDeep | UnsoundDie 
type typeTransformersGeneration = TTGYes | TTGNo | TTGExclusive
type args = 
   {
      mutable arg_currentMode           : mode;                 (* compilation mode *)
      mutable arg_updatabilityPolicy    : updatabilityPolicy;   (* update() keyword elimination policy *)
      mutable arg_updatePointsPolicy    : updatePointsPolicy;   (* update() keyword insertion policy *)
      mutable arg_effectPrintingPolicy  : effectPrintingPolicy; (* print effects policy *)
      mutable arg_patchDirectoryOut     : string;               (* directory where patches are stored *)
      mutable arg_versionDataInFile     : string;               (* file name to retrieve version data from *)
      mutable arg_versionDataOutFile    : string;               (* file name to store version data from *)
      mutable arg_dosuspect             : bool;                 (* whether or not to do suspect alias analysis *)
      mutable arg_dostaticconabs        : bool;                 (* whether to make con/abs functions static+inline *)
      mutable arg_dogvartypes           : bool;                 (* whether or not to do type introduction for gvars *)
      mutable arg_dopatchsplitting      : bool;                 (* split patches; by default is on *)
      mutable arg_gentts                : typeTransformersGeneration; (* generate type transformers; default is on (TTGYes) *)
      mutable arg_stoploops             : bool;                 (* emit code after extracting loops *)
      mutable arg_assembly              : unsoundnessAction;    (* what to do with inline assembly *)
      mutable arg_fromvoid              : unsoundnessAction;    (* what to do with casts from void* *)
      mutable arg_subtypes              : unsoundnessAction;    (* what to do with attempted physical or width subtypes under pointers *)
      mutable arg_restrictFuns          : bool;                 (* add functions used in continuation to restriction on __DSU_update() *) 
      mutable arg_restrictGvars         : bool;                 (* add global variables used in continuation to restriction on __DSU_update() *)
      mutable arg_restrictTypes         : bool;                 (* add types used in continuation to restriction on __DSU_update() *)
      mutable arg_vcEnabled             : bool;                 (* enable/disable VC effects *)
      mutable arg_pointstoEnabled       : bool;                 (* enable/disable the points-to analysis *)
      mutable arg_showProgress          : bool;                 (* print a line for each global the analysis is done with *)
      mutable arg_txIsolation           : bool;                 (* don't propagate effects inside tx *)
      mutable arg_funToPrint            : (string,unit) Hashtbl.t;    (* print effects for this fun *)
      mutable arg_wrapBlockingCalls     : bool;                 (* add start/end around blocking calls *)
      mutable arg_syncUpdates           : bool;                 (* overprovision delta and fwd effects *)
      mutable arg_focus                 : string option;        (* 'focus' effect printing on this effect only *)
      mutable arg_ttLocks               : bool;                 (* (don't) use locks to protect TTs *)
      mutable arg_checkins              : bool;                 (* (don't) insert check-ins *)
   }

(* Command line arguments stored here *)
let args = { 
      (* These are the defaults *)
      arg_currentMode = NormalMode;
      arg_updatabilityPolicy = UPNone;
      arg_updatePointsPolicy = UReturnonly;
      arg_effectPrintingPolicy = EPSolveOnly;
      arg_patchDirectoryOut = "";
      arg_versionDataInFile = "";
      arg_versionDataOutFile = "";
      arg_dosuspect = true;
      arg_dostaticconabs = true;
      arg_dogvartypes = false;
      arg_dopatchsplitting = false;
      arg_gentts = TTGYes;
      arg_stoploops = false;
      arg_assembly = UnsoundWarn;
      arg_fromvoid = UnsoundKillTypeShallow;
      arg_subtypes = UnsoundWarn;
      arg_restrictFuns = true;
      arg_restrictGvars = true;
      arg_restrictTypes = true;
      arg_vcEnabled = true;
      arg_pointstoEnabled = false;
      arg_showProgress = false;
      arg_txIsolation = true;
      arg_funToPrint = Hashtbl.create 3;
      arg_wrapBlockingCalls = false;
      arg_syncUpdates = false;
      arg_focus = None;
      arg_ttLocks = true;
      arg_checkins = true;
};;


(* Updatemain will use this as fd_extraopt to process Ginseng-specific arguments *)
let argsProcessor = [("--patchmode",
                       Arg.Unit (fun () -> args.arg_currentMode <- PatchMode),
                       "compile a patch \n");

                   ("--testmode",
                       Arg.Unit (fun () -> args.arg_currentMode <- TestMode),
                       "Run in test mode \n");

                   ("--printversiondata",
                       Arg.Unit (fun () -> args.arg_currentMode <- PrintVDMode),
                       "print the version data \n");

                   ("--updatability-tx",
                       Arg.Unit (fun () -> args.arg_updatabilityPolicy <- UPTx),
                       "only tx-functions and their callees can perform updates\n");

                   ("--update-points", 
                       Arg.String (fun s -> match s with
                                       "full"       -> args.arg_updatePointsPolicy <- UFull
                                     | "returnonly" -> args.arg_updatePointsPolicy <- UReturnonly
                                     | "eotx"       -> args.arg_updatePointsPolicy <- UEndOfTx
                                     | "none"       -> args.arg_updatePointsPolicy <- UNone
                                     | _ -> Updatedebug.abort (Updatedebug.error "Invalid argument for --update-points :'%s' ! \nIt must be [full | returnonly | none]\n" s)),
                       "=[full | returnonly | eotx | none] insert update() calls in the source code (default is returnonly) \n" ^
                                 "\t full : before each statement\n" ^
                                 "\t returnonly : before return keywords only\n" ^
                                 "\t eotx : at the end of transactions\n" ^
                                 "\t none : no update()s are inserted" );

                   ("--effect-printing", 
                       Arg.String (fun s -> match s with
                                       "none"         -> args.arg_effectPrintingPolicy <- EPNone
                                     | "functions"    -> args.arg_effectPrintingPolicy <- EPFunctions
                                     | "types"        -> args.arg_effectPrintingPolicy <- EPTypes
                                     | "solveonly"    -> args.arg_effectPrintingPolicy <- EPSolveOnly
                                     | "differential" -> args.arg_effectPrintingPolicy <- EPDifferential
                                     | "full"         -> args.arg_effectPrintingPolicy <- EPFull

                                     | _ -> Updatedebug.abort (Updatedebug.error "Invalid argument for --effect-printing :'%s' ! \nIt must be [full | differential | functions | types | solveonly | none]\n" s)),
                       "=[full | functions | types | none] insert inferred effects as comments in the source code (default is types) \n" ^
                                 "\t full : function labels and types \n" ^
                                 "\t differential : only print the difference in effects vs the previous/next statmenent \n" ^
                                 "\t types : types \n" ^
                                 "\t functions : function labels \n" ^
                                 "\t solveonly : solve for effects, but don't print them \n" ^
                                 "\t none : no effects are inserted" );

                   ("--assembly", 
                       Arg.String (fun s -> match s with
                                       "warn"         -> args.arg_assembly <- UnsoundWarn
                                     | "ignore"       -> args.arg_assembly <- UnsoundIgnore
                                     | "kill-shallow" -> args.arg_assembly <- UnsoundKillTypeShallow
                                     | "kill-deep"    -> args.arg_assembly <- UnsoundKillTypeDeep
                                     | "die"          -> args.arg_assembly <- UnsoundDie


                                     | _ -> Updatedebug.abort (Updatedebug.error "Invalid argument for --assembly :'%s' ! \nIt must be [ warn | ignore | die]\n" s)),
                       "=[ warn | ignore | die] what to do with inline assembly \n" );

                   ("--from-void", 
                       Arg.String (fun s -> match s with
                                       "warn"         -> args.arg_fromvoid <- UnsoundWarn
                                     | "ignore"       -> args.arg_fromvoid <- UnsoundIgnore
                                     | "kill-shallow" -> args.arg_fromvoid <- UnsoundKillTypeShallow
                                     | "kill-deep"    -> args.arg_fromvoid <- UnsoundKillTypeDeep
                                     | "die"          -> args.arg_fromvoid <- UnsoundDie


                                     | _ -> Updatedebug.abort (Updatedebug.error "Invalid argument for --from-void :'%s' ! \nIt must be [ warn | ignore | die]\n" s)),
                       "=[ warn | ignore | die] what to do when downcasting to T * from void * and [T] not in the void * effects \n" );

                   ("--patchdirectory-out", 
                       Arg.String (fun s -> args.arg_patchDirectoryOut <- s),
                       "directory where patches will be stored");

                   ("--versiondata-out", 
                       Arg.String (fun s -> args.arg_versionDataOutFile <- s),
                       "name of version data file output file");

                   ("--versiondata-in", 
                       Arg.String (fun s -> args.arg_versionDataInFile <- s),
                       "name of version data input file");

                   ("--no-suspect-alias", 
                       Arg.Unit (fun () -> args.arg_dosuspect <- false),
                       "don't do suspect alias analysis \n");

                   ("--no-static-conabs", 
                       Arg.Unit (fun () -> args.arg_dostaticconabs <- false),
                       "don't make con/abs functions static+inline \n");

                   ("--gvartypes", 
                       Arg.Unit (fun () -> args.arg_dogvartypes <- true),
                       "do type introduction for global variables \n");

                   ("--stop-loops", 
                       Arg.Unit (fun () -> args.arg_stoploops <- true),
                       "stop after extracting loops \n");

                   ("--no-patchsplit", 
                       Arg.Unit (fun () -> args.arg_dopatchsplitting <- false),
                       "don't split patches; by default, patches are split \n");

                   ("--no-update-restrict", 
                       Arg.String (fun s -> match s with 
                                     "f"  -> args.arg_restrictFuns <- false
                                    |"g"  -> args.arg_restrictGvars <- false
                                    |"t"  -> args.arg_restrictTypes <- false
                                    | _ -> Updatedebug.abort (Updatedebug.error "Invalid argument for --no-update-restrict :'%s' ! \nIt must be [f | g | t]+\n" s)),
                       "=[f | g | t] omit restriction on DSU_update (default is none, anything else is unsound !) \n" ^
                                 "\t f : omit functions \n" ^
                                 "\t g : omit global variables \n" ^
                                 "\t t : omit types " );    

                   ("--type-transformers", 
                       Arg.String (fun s -> match s with
                                       "yes"       -> args.arg_gentts <- TTGYes
                                     | "no"        -> args.arg_gentts <- TTGNo
                                     | "exclusive" -> args.arg_gentts <- TTGExclusive
                                     | _ -> Updatedebug.abort (Updatedebug.error "Invalid argument for --type-transformers :'%s' ! \nIt must be [yes | no | exclusive]\n" s)),
                       "=[yes | no | exclusive] generate type transformers when in patch generation mode (default is yes) \n" ^
                                 "\t yes : generate full patch (functions and type transformers) \n" ^
                                 "\t no : no type transformers generated, patch contains functions and type definitions only \n" ^
                                 "\t exclusive : generate type transformers only (no types and functions); useful for generating the .custom.c " );

                   ("--enable-debug", 
                       Arg.String (fun s -> Updatedebug.bugAction s true),
                       "Enable given debug printing flag \n");

                   ("--no-vc", 
                       Arg.Unit (fun () -> args.arg_vcEnabled <- false),
                       "disable VC\n");

                   ("--no-points-to", 
                       Arg.Unit (fun () -> args.arg_pointstoEnabled <- false),
                       "enable points-to analysis\n");

                   ("--show-progress", 
                       Arg.Unit (fun () -> args.arg_showProgress <- true),
                       "show analysis progress\n");

                   ("--no-tx-isolation", 
                       Arg.Unit (fun () -> args.arg_txIsolation <- false),
                       "propagate effects into tx body\n");

                   ("--annotate-fun", 
                       Arg.String (fun s -> Hashtbl.add args.arg_funToPrint s ()),
                       "print effects for this function\n");

                   ("--wrap-blockingcalls", 
                       Arg.Unit (fun () -> args.arg_wrapBlockingCalls <- true),
                       "wrap blocking calls\n");

                   ("--sync-updates", 
                       Arg.Unit (fun () -> args.arg_syncUpdates <- true),
                       "use sync update annotations on check-ins\n");

                   ("--focus", 
                       Arg.String (fun s -> args.arg_focus <- Some s),
                       "focus on this restriction only\n");

                   ("--no-tt-locks", 
                       Arg.Unit (fun () -> args.arg_ttLocks <- false),
                       "don't use locks for type transformers\n");

                   ("--no-checkins", 
                       Arg.Unit (fun () -> args.arg_checkins <- false),
                       "don't insert check-ins\n");

];;
