(* 
 *
 * Copyright (c) 2004-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *  Michael Hicks       <mwh@cs.umd.edu> 
 *  Gareth Stoyle       <gareth.stoyle@cl.cam.ac.uk> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



(* debug code *)

open Cil


type bugType = 
   BTAlways
 | BTConabsinsertion
 | BTWrappers
 | BTUpdatecilutil
 | BTPtraliastracking
 | BTSubtype
 | BTNamedtypes
 | BTExternaltypeselim
 | BTSizeofelim  (* NB. make this true to non-update types passed to sizeof() *)
 | BTTestvisitors
 | BTUpdatemain 
 | BTUpdatedebug 
 | BTSetTypeNonUpdatable
 | BTIndirection
 | BTUpdatepoints
 | BTStatistics
 | BTExternalvoidstarfunctions
 | BTLoops
 | BTAnalysis
 | BTPatchcomp
 | BTDeltasolver
 | BTMusolver
 | BTPatchgen
 | BTSuspect
 | BTEffects
 | BTGvartypes
 | BTPatchsplit;;

let defaultBugTypeIsEnabled = function
   BTAlways -> true
(* Turn on debug info for each module *)
 | BTConabsinsertion           -> false
 | BTWrappers                  -> false
 | BTUpdatecilutil             -> false
 | BTPtraliastracking          -> false
 | BTSubtype                   -> false
 | BTNamedtypes                -> false
 | BTExternaltypeselim         -> false
 | BTSizeofelim                -> false
 | BTTestvisitors              -> false
 | BTUpdatemain                -> false
 | BTUpdatedebug               -> false
 | BTIndirection               -> false
 | BTUpdatepoints              -> false
 | BTExternalvoidstarfunctions -> false
 | BTLoops                     -> false
 | BTAnalysis                  -> false
 | BTPatchcomp                 -> false
 | BTDeltasolver               -> false
 | BTMusolver                  -> false
 | BTPatchgen                  -> false
 | BTSuspect                   -> false
 | BTEffects                   -> false
 | BTGvartypes                 -> false
 | BTPatchsplit                -> false

(* Debug a feature (which may span modules) *)
 | BTSetTypeNonUpdatable       -> false
 | BTStatistics                -> true
;;

let bugTypeIsEnabled = ref defaultBugTypeIsEnabled

let bugAction (bug:string) (action:bool) =
  let oldf = !bugTypeIsEnabled in
  let newf = 
    match bug with
      "Conabsinsertion" -> 
	(function 
	    BTConabsinsertion -> action
	  | x -> oldf x)
    | "Wrappers" -> 
	(function 
	    BTWrappers -> action
	  | x -> oldf x)
    | "Updatecilutil" -> 
	(function 
	    BTUpdatecilutil -> action
	  | x -> oldf x)
    | "Ptraliastracking" -> 
	(function 
	    BTPtraliastracking -> action
	  | x -> oldf x)
    | "Subtype" -> 
	(function 
	    BTSubtype -> action
	  | x -> oldf x)
    | "Namedtypes" -> 
	(function 
	    BTNamedtypes -> action
	  | x -> oldf x)
    | "Externaltypeselim" -> 
	(function 
	    BTExternaltypeselim -> action
	  | x -> oldf x)
    | "Sizeofelim" -> 
	(function 
	    BTSizeofelim -> action
	  | x -> oldf x)
    | "Testvisitors" -> 
	(function 
	    BTTestvisitors -> action
	  | x -> oldf x)
    | "Updatemain" -> 
	(function 
	    BTUpdatemain -> action
	  | x -> oldf x)
    | "Updatedebug" -> 
	(function 
	    BTUpdatedebug -> action
	  | x -> oldf x)
    | "SetTypeNonUpdatable" -> 
	(function 
	    BTSetTypeNonUpdatable -> action
	  | x -> oldf x)
    | "Indirection" -> 
	(function 
	    BTIndirection -> action
	  | x -> oldf x)
    | "Updatepoints" -> 
	(function 
	    BTUpdatepoints -> action
	  | x -> oldf x)
    | "Statistics" -> 
	(function 
	    BTStatistics -> action
	  | x -> oldf x)
    | "Externalvoidstarfunctions" -> 
	(function 
	    BTExternalvoidstarfunctions -> action
	  | x -> oldf x)
    | "Loops" -> 
	(function 
	    BTLoops -> action
	  | x -> oldf x)
    | "Analysis" -> 
	(function 
	    BTAnalysis -> action
	  | x -> oldf x)
    | "Patchcomp" -> 
	(function 
	    BTPatchcomp -> action
	  | x -> oldf x)
    | "Deltasolver" -> 
	(function 
	    BTDeltasolver -> action
	  | x -> oldf x)
    | "Musolver" -> 
	(function 
	    BTMusolver -> action
	  | x -> oldf x)
    | "Patchgen" -> 
	(function 
	    BTPatchgen -> action
	  | x -> oldf x)
    | "Suspect" -> 
	(function 
	    BTSuspect -> action
	  | x -> oldf x)
    | "Effects" -> 
	(function 
	    BTEffects -> action
	  | x -> oldf x)
    | "Gvartypes" -> 
	(function 
	    BTGvartypes -> action
	  | x -> oldf x)
    | "Patchsplit" -> 
	(function 
	    BTPatchsplit -> action
	  | x -> oldf x)
    | _ -> failwith "Unknown debugging option" in
  bugTypeIsEnabled := newf

let shouldPrintErrorMsg btl = 
   List.exists !bugTypeIsEnabled btl;;

let abort = Commondebug.abort;;
let error = Commondebug.error;;
let bug   = Commondebug.bug;;


let log (btl : bugType list) (fmt : ('a,unit,Pretty.doc) format) : 'a = 
   Commondebug.log (shouldPrintErrorMsg btl) fmt

let ppType t = 
   (* if the attributes contain any effects, include the EFFECT keyword on the arrow*)
   let string_of_effects attrs = 
      let attributes_with_effects = Cil.filterAttributes Updateconfig.effectAttrName attrs in
      let eff_print_fold e acc = 
         begin match e with 
               Attr(aname, [AInt(n)]) when aname = Updateconfig.effectAttrName -> (" EFFECT (" ^ (string_of_int n)^ ") ") ^ acc
            | _ ->" EFFECT (?) " ^ acc
         end in
         List.fold_right eff_print_fold attributes_with_effects ""
   in
      Commoncilutil.prettyPrintType ~effect_to_string:string_of_effects t

let ppInstrList (il : Cil.instr list) : unit = 
   List.iter (fun i -> ignore(Errormsg.log "%a" d_instr i)) il;
   ignore(Errormsg.log "\n")

let d_file (f : Cil.file) : unit =
    Cil.iterGlobals f (fun g -> ignore (log [BTAlways] "%a\n" d_global g));; 
(* Cil.iterGlobals f (fun g -> match g with *)
(*                          GVar(vi,{init = Some (CompoundInit (t,oi))},_) when vi.vname = "check" -> ignore (log [BTAlways] " %a\n"  d_plaintype t) *)
(*                       | _ -> () *)
(*                   );; *)


