(* 
 *
 * Copyright (c) 2004-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *  Gareth Stoyle       <gareth.stoyle@cl.cam.ac.uk> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



(* Insert call to updatelib's initialisation function *)

open Cil

let insertCallToUpdatelibInit (file:Cil.file) (checkinIdToCapMap : Updatepoints.checkinIdToCapMap_t) : unit =
   let locateMain_fold (g:Cil.global) ((acc, found_main) : Cil.global list * bool) : (Cil.global list * bool) = 
      match g with 
         GFun(fd,loc) when fd.svar.vname = "main" ->
            (* located main: create extern declaration of initialisation function and insert call to updatelib's 
               initialisation function as first as first statement,
               followed by calls to add checkin point caps *)
            let init_fun_vi = Cil.makeGlobalVar Updateconfig.updatelibInitFunName Updateconfig.updatelibInitFunType in
            let addcap_vi = Cil.makeGlobalVar Updateconfig.addcapFunName Updateconfig.addcapFunType in
            let addcap_instrs = 
              if Updateargs.args.Updateargs.arg_checkins
              then Updatepoints.genAddcapInstrList checkinIdToCapMap addcap_vi 
              else []
            in
            init_fun_vi.vstorage <- Cil.Extern;
            addcap_vi.vstorage <- Cil.Extern;
            fd.sbody.bstmts <- (Commoncilutil.mkStmt ( Instr (Call(None, Lval (var init_fun_vi), [], Cil.locUnknown)::addcap_instrs) ))
                               ::fd.sbody.bstmts;
            (* if arg_checkins is false, don't insert addcap()'s *)
            (if Updateargs.args.Updateargs.arg_checkins
             then ((GVarDecl( init_fun_vi, Cil.locUnknown )) ::(GVarDecl( addcap_vi, Cil.locUnknown )) :: g :: acc), true
             else ((GVarDecl( init_fun_vi, Cil.locUnknown ))                                           :: g :: acc), true
            )
       | _ -> (g::acc), found_main
   in

   let (globals', found_main) = List.fold_right locateMain_fold file.globals ([],false) in 
   if not found_main then
      Updatedebug.abort (Updatedebug.error "insertCallToUpdatelibInit: Didn't find main, so couldn't insert call to updatelib's initialisation function\n")
   
