/* Patch generation test suite 
 *
 * Base version
 */

#pragma __DSU_malloc("malloc");

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>


/********************************************************************
* Items to store in list                                            *
*********************************************************************/
struct T {
	int x;
};

void printT(struct T* pt) 
{
   printf("t.x=%d\n",pt->x);
   return;
}



/********************************************************************
* List definition and functions                                     *
*********************************************************************/
struct Tlist {
  struct T item;
  struct Tlist* next;
};

struct Tlist* consT(struct T* pt, struct Tlist* pl)
{
  struct Tlist* l = malloc(sizeof(struct Tlist));
  l->next = pl;
  l->item = *pt;

  return l;
}

void iterT(void(*f)(struct T*), struct Tlist* pl)
{
  while(pl)
    {
      f(&pl->item);
      pl = pl->next;
    }
  return;
}


extern int update_requested;



/********************************************************************
* The main program loop will be extracted                                            *
*********************************************************************/
#pragma __DSU_loop("MAINLOOP")

void print_list(struct Tlist* pl) 
{
  printf("*** interactive_update version 0: list elements have one member, .x\n");
  iterT(printT,pl);
}

void interactive_update(struct Tlist* pl) 
{

  /* Do an update */
  printf("*** interactive_update version 0: Hit ENTER to request update\n");
  getchar();
  printf("*** Start update\n");
  /* Usually signalling an available update is triggered by sending a 
     SIGUSR2 to the process, but in this case we want to spare the user of doing that */
  update_requested = 1;
  //__DSU_update("");
  printf("*** End update \n");
}

int global_version = 0;

struct Tlist* pl = NULL;

void * child_thread(void * arg)
{

  thread_register("child");

  while(1) {
  __DSU_CHECKIN:{
      print_list(pl);

      sleep(1);
    }
  }
  thread_deregister("child");
}


int main()
{
  struct T t;
  int i;
  pthread_t child;
  
  /* Create a list */
  for(i=10; i>0; i--) {
    t.x = i;
    pl = consT(&t,pl);
  }

  i = 0;

  thread_register("main");


  pthread_create(& child, 
                 NULL, 
                 child_thread,
                 NULL);

  MAINLOOP:while(1) {  

  __DSU_CHECKIN: {
      printf("*** MAINLOOP: main() version 0, global_version = %d\n", global_version);

      interactive_update(pl);
      
      i++;
      if (i >= 3)
        break;
    }
  }

  thread_deregister("main");

  return 0;
}
