/* Patch generation test suite 
 *
 * Changes: added extra field (y) to elements stored in list
 */

#pragma __DSU_malloc("malloc");

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>


/********************************************************************
* Items to store in list                                            *
*********************************************************************/
struct T {
	int x;
   int y;   /* new field */
};

void printT(struct T* pt) 
{
   printf("t.x=%d, t.y=%d\n",pt->x, pt->y);
   return;
}



/********************************************************************
* List definition and functions                                     *
*********************************************************************/
struct Tlist {
  struct T item;
  struct Tlist* next;
};

struct Tlist* consT(struct T* pt, struct Tlist* pl)
{
  struct Tlist* l = malloc(sizeof(struct Tlist));
  l->next = pl;
  l->item = *pt;

  return l;
}

void iterT(void(*f)(struct T*), struct Tlist* pl)
{
  while(pl)
    {
      f(&pl->item);
      pl = pl->next;
    }
  return;
}





/********************************************************************
* The main program loop                                             *
*********************************************************************/
void loop(struct Tlist* pl) 
{
  iterT(printT,pl);

  /* Do an update */
  getchar();
  printf("*** Start update\n\n\n");
  __DSU_update();
  printf("*** end update \n\n\n");

  loop(pl); /* tail recursive call */
}



int main()
{
  struct Tlist* pl = 0;
  struct T t;
  int i;
  
  /* Create a list */
  for(i=0; i<10; i++) {
    t.x = i;
    pl = consT(&t,pl);
  }
  
  loop(pl);
  return 0;
}
