#!/bin/bash
    
function die ()
{
    echo $1
    exit 1
}

[ $# -ge 6 ]  || die "Usage: $0 <base program name> <start version> <end version> <output directory> <absolute path to cilly bin directory> <absolute path to runtime directory>"


PROGRAM=$1
START_VERSION=$2
END_VERSION=$3
OUTPUT_DIRECTORY=$4
CILLY_BIN_DIRECTORY=$5
RUNTIME_DIRECTORY=$6

[ -d $OUTPUT_DIRECTORY ] || die "Output directory '$OUTPUT_DIRECTORY' doesn't exist"

let "number_of_versions = $END_VERSION - $START_VERSION + 1"

[ $# -ge 4 ]  || die "end_version must be greater or equal than start_version + 1"


[ -f PREAMBLE ]  || die "File 'PREAMBLE' doesn't exist"
[ -f POSTAMBLE ]  || die "File 'POSTAMBLE' doesn't exist"

OUTPUT_FILE="$OUTPUT_DIRECTORY/Makefile"

echo "################ Automatically generated makefile #################" > $OUTPUT_FILE
echo "################ Generated by $* " >> $OUTPUT_FILE

# set CILLY_BIN_DIR 
# and RUNTIME_DIR 


echo "CILLY_BIN_DIR = $CILLY_BIN_DIRECTORY" >> $OUTPUT_FILE
echo "RUNTIME_DIR = $RUNTIME_DIRECTORY" >> $OUTPUT_FILE

echo "cat PREAMBLE >> $OUTPUT_FILE"
cat PREAMBLE >> $OUTPUT_FILE


STREAM=""

# first create symlinks 
for (( virtual_version = 0 ; virtual_version < $number_of_versions ;  virtual_version += 1 ));do

    let "actual_version = $virtual_version+$START_VERSION"

    SOURCE_FILE=`pwd`"/$PROGRAM-$actual_version.c"
    DEST_FILE="$OUTPUT_DIRECTORY/$PROGRAM-$virtual_version.c"

    [ -f $SOURCE_FILE ] || die "Can't find source file $SOURCE_FILE"

    echo "ln -s  $SOURCE_FILE $DEST_FILE"
    ln -s  $SOURCE_FILE $DEST_FILE
done

# copy the Makefile.tmpl over
    [ -f $OUTPUT_DIRECTORY/Makefile.tmpl ] || (echo " CILLY = $CILLY_BIN_DIRECTORY/cilly" >  $OUTPUT_DIRECTORY/Makefile.tmpl;cat Makefile.tmpl >>$OUTPUT_DIRECTORY/Makefile.tmpl )

# then Makefile rules
for (( virtual_version = 0 ; virtual_version < $number_of_versions - 1;  virtual_version += 1 ));do

    let "old_v=$virtual_version"
    let "new_v=$virtual_version + 1"

#  0/   1/  ...   , directories for storing patches
    PATCH_DIRECTORY="$OUTPUT_DIRECTORY/$old_v"
    echo "mkdir $PATCH_DIRECTORY"
    mkdir $PATCH_DIRECTORY || die "Can't create patch output directory $PATCH_DIRECTORY"

    [ -f $PATCH_DIRECTORY/Makefile ] || (cd $PATCH_DIRECTORY; ln -s ../Makefile.tmpl Makefile )


# %.0-1.patch.c : %-0.c %-1.c
    echo $old_v $new_v | awk '// {printf("%%.%d-%d.patch.c : %%-%d.c %%-%d.c\n", $1, $2, $1, $2);}' >> $OUTPUT_FILE
# \t	CILLY_DONT_COMPILE_AFTER_MERGE=1  $(CILLY)  $(PATCHGEN_CILLY_FLAGS) --patchfile-out=$@ $*-0.c $*-1.c || /bin/true
    echo $PROGRAM $old_v $new_v| awk '// {printf("\tCILLY_DONT_COMPILE_AFTER_MERGE=1  $(CILLY)  $(PATCHGEN_CILLY_FLAGS) --patchfile-out=%d/%s.patch.c $*-%d.c $*-%d.c || /bin/true\n", $2, $1, $2, $3);}' >> $OUTPUT_FILE
# \t ln -s versiondata-0.vd 0/versiondata-0.vd
    echo $old_v $PROGRAM | awk '// {printf("\t cd %d && ln -sf ../%s-%d.vd %s-0.vd\n", $1, $2, $1, $2);}' >> $OUTPUT_FILE
# \t ln -s versiondata-1.vd 0/versiondata-1.vd
    echo $new_v $old_v $PROGRAM | awk '// {printf("\t cd %d && ln -sf ../%s-%d.vd %s-1.vd\n", $2, $3, $1,  $3);}' >> $OUTPUT_FILE

    echo >> $OUTPUT_FILE
    echo >> $OUTPUT_FILE

STREAM="$STREAM $PROGRAM.$old_v-$new_v.patch.c"

done

# all must come as first target, to build .exe first
echo "all : $PROGRAM.exe $PROGRAM.fullpatch" >> $OUTPUT_FILE

echo >> $OUTPUT_FILE

echo "$PROGRAM.fullpatch : $STREAM" >> $OUTPUT_FILE

for (( virtual_version = 0 ; virtual_version < $number_of_versions - 1;  virtual_version += 1 ));do

    let "old_v=$virtual_version"
    let "new_v=$virtual_version + 1"    

#	\t cd $*/0 && make
echo $old_v | awk '// {printf("\tcd %d && make\n", $1);}' >> $OUTPUT_FILE
    
done

echo >> $OUTPUT_FILE
echo >> $OUTPUT_FILE



echo "cat POSTAMBLE >> $OUTPUT_FILE"
cat POSTAMBLE >> $OUTPUT_FILE

echo 
echo  "Success ! Now cd into $OUTPUT_DIRECTORY and type 'make' "
