/* 
 *
 * Copyright (c) 2004-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *  Gareth Stoyle       <gareth.stoyle@cl.cam.ac.uk> 
 *  Manuel Oriol        <moriol@inf.ethz.ch> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 */



#ifndef DSU_DEBUG_H
#define DSU_DEBUG_H

#include <stdio.h>

#include "dsu_config.h"

#if DSU_ENABLE_DEBUG == 1
#define DSU_DEBUG(...) do { \
  if (__DSU_debug_file) \
  { \
    fprintf(__DSU_debug_file,"DSU DEBUG: ");\
    fprintf(__DSU_debug_file,__VA_ARGS__); \
    fflush(__DSU_debug_file); \
  } \
} \
while(0);

#define DSU_DUMP(...) do { \
  if (__DSU_debug_file) \
  { \
    fprintf(__DSU_debug_file,__VA_ARGS__); \
  } \
} \
while(0);

#else
#define DSU_DEBUG(...)
#define DSU_DUMP(...)
#endif // DSU_ENABLE_DEBUG

#if DSU_ENABLE_ERROR == 1
#define DSU_ERROR(...) do { \
  if (__DSU_debug_file) \
  { \
    fprintf(__DSU_debug_file,"DSU ERROR: ");\
    fprintf(__DSU_debug_file,__VA_ARGS__); \
    fflush(__DSU_debug_file); \
  } \
} \
while(0);
#define DSU_ERROR_NOPREFIX(...) do { \
  if (__DSU_debug_file) \
  { \
    fprintf(__DSU_debug_file,__VA_ARGS__); \
    fflush(__DSU_debug_file); \
  } \
} \
while(0);


#else
#define DSU_ERROR(...)
#define DSU_ERROR_NOPREFIX(...)
#endif // DSU_ENABLE_ERROR

#if DSU_ENABLE_FDEBUG == 1
#define DSU_FDEBUG(...) do { \
  if (__DSU_debug_file) \
  { \
    fprintf(__DSU_debug_file,"DSU FDEBUG: ");\
    fprintf(__DSU_debug_file,__VA_ARGS__); \
    fflush(__DSU_debug_file); \
  } \
} \
while(0);

#else
#define DSU_FDEBUG(...)
#endif // DSU_ENABLE_FDEBUG


extern FILE *__DSU_debug_file;

#endif  //DSU_DEBUG_H
