/* 
 *
 * Copyright (c) 2005-2009, 
 * 
 *  Manuel Oriol        <moriol@inf.ethz.ch> 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 */



#ifndef FGC_H
#define FGC_H

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>



/*
 * Structure for all the elements of the list.
 */
struct __DSU_FVersionElem {
  int vNumber;
  int nRef;
  char *libName;
  void *fct;
  void *handler;
  struct __DSU_FVersionElem* next;
  struct __DSU_FVersionElem* prev;
};

/*
 * Creates a __DSU_FVersionElem
 */
struct __DSU_FVersionElem* __DSU_consFVersionElem(char *libName, void *handler, int vNumber, void *fct);

/*
 * Structure for the head of the list.
 */ 
struct __DSU_FVersionList{
  char *baseFunName;
  struct __DSU_FVersionElem *first;
  void *activeFunction;
  unsigned int oldVersionsTotalCount;
  unsigned int length;
};


/*
 * Creates an empty __DSU_FVersionList
 */
struct __DSU_FVersionList* __DSU_consFVersionList(char *baseFunName);

/*
 * adds an element at the end of the linkedlist
 * MO: NO NEED TO BE THREAD-SAFE, AT UPDATE-TIME
 * we add it at the beginning for the reason that it is likely that
 * most recent versions will probably be used more often.
 */
void __DSU_addFVersionList(struct __DSU_FVersionList *l,char *name, unsigned int f_versionNumber, unsigned int p_versionNumber);


/*
 * Prints the state of a version list on stderr.
 * MO: SHOULD BE THREAD-SAFE
 */
void __DSU_printFVersionList(struct __DSU_FVersionList *l);

/*
 * List for functions to be GCed
 */
struct __DSU_Fct2Clean{
  struct __DSU_FVersionList *l;
  struct __DSU_Fct2Clean *next;

};

/*
 * add to a __DSU_Fct2Clean
 */
void __DSU_addFct2Clean(struct __DSU_Fct2Clean *cleaner,struct __DSU_FVersionList *l);

/*
 * Variable used to see if a GC has to be triggered.
 */
extern int __DSU_TRIGGER_FGC; 
extern struct __DSU_Fct2Clean __DSU_FGARBAGE;

/*
 * this function makes the effective garbage collection
 */
void __DSU_FGC();

/*
 * This loads the function.
 * returns:
 *    -1 if it was a failure.
 *    0 otherwise.
 */
int __DSU_load(const char *libname, unsigned int versionNumber);
/*
 * This unloads the old function.
 * returns:
 *    -1 if it was a failure.
 *    0 if the unloading was effectively made.
 */
int __DSU_unload(struct __DSU_FVersionList *l, unsigned int versionNumber);
/*
 * This locks the function.
 * MO: SHOULD BE THREAD-SAFE
 * returns:
 *    -1 if it was a failure.
 *    n: number of locks taken on this function (including this one).
 */
int __DSU_lock(struct __DSU_FVersionList *l, unsigned int versionNumber);

/*
 * This locks the function.
 * MO: SHOULD BE THREAD-SAFE
 * returns:
 *    -1 if it was a failure.
 *    n: number of locks still taken on this function (after this one is released).
 */
int __DSU_unlock(struct __DSU_FVersionList *l, unsigned int versionNumber);

#endif // FGC_H
