/* 
 *
 * Copyright (c) 2008-2009, 
 * 
 *  Eric Hardisty       <hardisty@cs.umd.edu>
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 */



#ifndef MACHNDEP_H
#define MACHNDEP_H

/*********************************************************
 *                                                       *
 * The OS/platform dependent part of the runtime system. *
 *                                                       *  
 *********************************************************
 */

#include "dsu_config.h"

/* Include the appropriate platform specific header depending
   on the value set as ARCH in the Makefile. */
#ifdef vxworks
#include "machndep_vxworks.h"
#endif 

#ifdef linux_x86 
#include "machndep_linux_x86.h" 
#endif 

/* Initializes the low-level part of the runtime system. 
 * Called by __DSU_init_handler()  
 */

extern void __DSU_machndep_init();


/*******************************/
/***** Dynamic loading *********/
/*******************************/


/* try to load the patch;
   if OK, return the handle, fill out metadata; 
   if failure, return NULL 
 
 * note that the update is not completed; apply_patch does that
*/
extern handle_t * __DSU_machndep_load_patch(int version, metadata_t * type_metadata, metadata_t * def_metadata);

/* effectively apply the patch: do fun redirection and invoke state transformer */
extern int __DSU_machndep_apply_patch(handle_t handle, int version);

/*******************************/
/***** Timing ******************/
/*******************************/

/* get crt. time */
extern void __DSU_machndep_get_perf_time(perf_time_t * tm);

/* convert abstract time into usecs so we can do addition/subtraction, etc. */
extern long long  __DSU_machndep_usec_of_perf_time(perf_time_t tm);

/* nanosleep */
extern void __DSU_machndep_sleep_nsec(long nsec);


/*  Name of dynamic patch file to load; changes from program to program, so
 *  we can't hardcode it; rather, the 'make' process that builds the executable and 
 *  the patch will take care of it.
 */
extern char * __DSU_libname;
extern char * __DSU_progname;

/*******************************/
/****** Threads ****************/
/*******************************/

#ifdef DSU_USE_THREADS
extern int __DSU_mutex_init(__DSU_mutex_t * mtx);
extern int __DSU_mutex_lock(__DSU_mutex_t * mtx);
extern int __DSU_mutex_trylock(__DSU_mutex_t * mtx);
extern int __DSU_mutex_unlock(__DSU_mutex_t * mtx);
extern __DSU_thread_t __DSU_thread_self();
extern int __DSU_cond_init(__DSU_cond_t * cond);
extern int __DSU_cond_wait(__DSU_cond_t * cond, __DSU_mutex_t * mtx);
extern int __DSU_cond_broadcast(__DSU_cond_t * cond);
#endif // DSU_USE_THREADS

#endif  //MACHNDEP_H
