/* 
 *
 * Copyright (c) 2006-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 */



#ifndef MT_H
#define MT_H

#include "dsu_config.h"
#include "dsu_debug.h"
#include "machndep.h"

#ifdef DSU_USE_THREADS

#define THREADS_SYNCED 0

/* DSU multithreading (MT) API */

/* Initializes MT API. Called by __DSU_init_handler()  */
extern void __DSU_mt_init();

/* a thread calls check_in(checkin_id) to 
 * check in its capability; the call is generated
 * automatically, based on 
 * __DSU_checkin: {} 
 * annotations and the (checkin_id -> cap) map
 */
/* checkin-id : sid * 1000 + [1|2] */
typedef int cid_t;

extern void __DSU_addcap(cid_t checkin_id, const char * delta, const char * fwd, const char * bck, const char * info, const char * checkin_label);
extern void __DSU_checkin(int checkin_id);

/*
 * thread_register() tells the MT API about the existence of this thread  
 * thread_deregister() tells the MT API the current thread doesn't exist any more
 */
extern void thread_register(const char * s);
extern void thread_deregister(const char * s);

/* 
 * conflicts_with_any_thread() checks changed_symbols
 * against the capability of each thread
 * if there's no conflict, it returns 0, 
 * otherwise it returns 1
 */
extern int  conflicts_with_any_thread(char ** changed_types, char ** changed_defs);

/* thread local data */
typedef struct {
  /* index in thread array */
  int index;
} tl_data;

#endif // DSU_USE_THREADS

#endif  //MT_H
