/* 
 *
 * Copyright (c) 2006-2009, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 */



/* Map implementation. Stores (char * key, void * value) pairs  */
#include <stdlib.h>
#include <stdio.h>
#include <search.h>
#include <string.h>
#include "tinymap.h"

#ifdef DSU_USE_THREADS

extern void tdestroy (void *root, void (*free_node)(void *nodep));

int mCompar(const void * v1, const void * v2)
{
  ENTRY * e1, * e2;
  e1 = (ENTRY *)v1;
  e2 = (ENTRY *)v2;

  if (*((__DSU_thread_t *)e1->key) < *((__DSU_thread_t *)e2->key)) return -1;
  if (*((__DSU_thread_t *)e1->key) > *((__DSU_thread_t *)e2->key)) return  1;
  return 0;
}

void mFreeNode(void * nodep)
{
  ENTRY * e = *(ENTRY **)nodep;

/*   printf("mFreeNode(%d):%s\n", *e->key, (char *)e->data); */

  free(e->key);
  free(e);
}

void * mCreateMap() 
{ 
  struct tree_root * tr_handle = calloc(1, sizeof(struct tree_root));
  
  return tr_handle;
}

void   mAdd(void * mapp, __DSU_thread_t key, void * value)
{ 
  ENTRY * e = malloc(sizeof(ENTRY));

  if (e == NULL)
    return;

  e->key = (char *)malloc(sizeof(__DSU_thread_t));
  *((__DSU_thread_t *)e->key) = key;
  e->data = value;

  tsearch(e, &(((struct tree_root *)mapp)->rootp), mCompar);
}

void * mFind(void * mapp, __DSU_thread_t key)
{ 
  ENTRY ** ret;

  ENTRY * e = malloc(sizeof(ENTRY));

  if (e == NULL)
    return NULL;

  e->key = (char *)malloc(sizeof(__DSU_thread_t));
  *((__DSU_thread_t *)e->key) = key;
  e->data = NULL;

  ret = tfind(e, &(((struct tree_root *)mapp)->rootp), mCompar);

  if (ret == NULL)
    return NULL;
  else
    return (*ret)->data;
}

void   mRemove(void * mapp, __DSU_thread_t key)
{
  ENTRY * e = malloc(sizeof(ENTRY));

  if (e == NULL)
    return;

  e->key = (char *)malloc(sizeof(__DSU_thread_t));
  *((__DSU_thread_t *)e->key) = key;
  e->data = NULL;

  tdelete(e, &(((struct tree_root *)mapp)->rootp), mCompar);
}

void   mDeleteMap(void * mapp)
{
  tdestroy(((struct tree_root *)mapp)->rootp, mFreeNode);

  free(mapp);
}


static void print_one_node(const void *nodep, const VISIT which, const int depth)
{
  ENTRY * e = *(ENTRY **)nodep;

/*   printf("e:%p key: %p (%d), data %p (%s)\n", e, e->key, *e->key, e->data, (char *)e->data); */
  printf("e: key: %lx, data %p\n", *((__DSU_thread_t *)e->key), e->data);
}


void mPrint(void *mapp)
{
  twalk(((struct tree_root *)mapp)->rootp, print_one_node);
}


#endif // DSU_USE_THREADS

/*  TEST */

/* int main() */
/* { */
/*   int i; */
/*   __DSU_thread_t keys[]={0xb7eec6b0, */
/*                     0xb7eebba0, */
/*                     0xb74eaba0, */
/*                     0xb60e8ba0, */
/*                     0xb6ae9ba0}; */

/*   void * root = mCreateMap(); */
/*   char * values[] = {"zero", "one","two","three","four"}; */
/*   for (i = 0; i < 5; i++) { */
/*     mAdd(root, keys[i], values[i]); */
/*   } */

/*   printf("map contents:\n"); */
/*   mPrint(root); */

/*   for (i = 0; i <= 5; i++) { */
/*     printf("mfind(%lx):%s\n", keys[i], (char *)mFind(root,keys[i])); */
/*   } */

/*   mRemove(root,keys[0]); */
/*   mRemove(root,keys[1]); */
/*   mRemove(root,keys[2]); */
/*   mRemove(root,keys[3]); */
/*   mRemove(root,keys[4]); */

/*   printf("map contents:\n"); */
/*   mPrint(root); */

/*   mAdd(root, keys[1], values[0]); */

/*   printf("map contents:\n"); */
/*   mPrint(root); */

/*   mDeleteMap(root); */

/*   return 0; */
/* } */
